# @Author : Donglai
# @Time   : 7/12/2020 2:49 AM
# @Email  : dma96@atmos.ucla.edu donglaima96@gmail.com

from datetime import datetime, timedelta
import mechanize
import os
from .config import CONFIG
import pandas as pd
from ..predict.realtime_read import pdframe_to_mat

def load(time_now = None,date_history = 30):
    """
    This is the real time downloader in kyoto website
    'http://wdc.kugi.kyoto-u.ac.jp/dstae/index.html'

    """
    if not os.path.exists(CONFIG['local_data_dir']):
        os.makedirs(CONFIG['local_data_dir'])

    if time_now is None:
        date_now = datetime.utcnow()
    else:
        date_now = time_now

    start_time = date_now - timedelta(days = date_history)
    end_time = date_now


    sCent = int(start_time.year / 100)
    sTens = int((start_time.year - sCent * 100) / 10)
    sYear = int(start_time.year - sCent * 100 - sTens * 10)
    sMonth = start_time.strftime("%m")
    eCent = int(end_time.year / 100)
    eTens = int((end_time.year - eCent * 100) / 10)
    eYear = end_time.year - eCent * 100 - eTens * 10
    eMonth = end_time.strftime("%m")

    dst_br = mechanize.Browser()
    dst_br.set_handle_robots(False)  # no robots
    dst_br.set_handle_refresh(False)  # can sometimes hang without this
    dst_br.addheaders = [('User-agent',
                      'Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.1) Gecko/2008071615 Fedora/3.0.1-1.fc9 Firefox/3.0.1')]
    dst_br.open(CONFIG['remote_data_dir'])

    dst_br.form = list(dst_br.forms())[0]

    # fill out the page fields
    dst_br.form.find_control('SCent').value = [str(sCent)]
    dst_br.form.find_control('STens').value = [str(sTens)]
    dst_br.form.find_control('SYear').value = [str(sYear)]
    dst_br.form.find_control('SMonth').value = [sMonth]
    dst_br.form.find_control('ECent').value = [str(eCent)]
    dst_br.form.find_control('ETens').value = [str(eTens)]
    dst_br.form.find_control('EYear').value = [str(eYear)]
    dst_br.form.find_control('EMonth').value = [eMonth]

    dst_br.form.find_control('Output').value = ['DST']
    dst_br.form.find_control('Out format').value = ['IAGA2002']
    dst_br.form.find_control('Email').value = "donglaima96@gmail.com"

    response = dst_br.submit()
    lines = response.readlines()
    dst_time = []
    dst_name = 'DST_realtime.txt'
    dst_txt_name = os.path.join(CONFIG['local_data_dir'], dst_name)
    dst_file = open(dst_txt_name,'w')
    for l in lines:

        clean_line = l.strip().decode("utf-8")

        if clean_line[-1] is not '|':
            dst_time.append(clean_line.split("\s+")[0])
            dst_file.write(clean_line)
            dst_file.write('\n')
    dst_file.close()
    dst_all = pd.read_csv(dst_txt_name, header=None, sep='\s+',
                          parse_dates=[[0, 1]], na_values=99999.99)
    dst_pdframe = dst_all.iloc[:, [0, 2]]
    dst_pdframe.columns = ['date', 'dst_kyoto']
    dst_pdframe.set_index('date', inplace=True, drop=True)
    pdframe_to_mat(dst_pdframe,CONFIG['real_data_dir'],res = '60m')



    return dst_pdframe


# @Author : Donglai
# @Time   : 7/12/2020 1:57 AM
# @Email  : dma96@atmos.ucla.edu donglaima96@gmail.com



import numpy as np
from scipy.io import loadmat,savemat
import os
from datetime import datetime, timedelta
from scipy.interpolate import interp1d
import tensorflow as tf
import pandas as pd
import matplotlib.pyplot as plt
import tempfile
from .config import CONFIG
from .. import dst_kyoto
from .. import ae
from .. import ace
from .. import dst

from mpl_toolkits.axes_grid1.inset_locator import inset_axes
import pkgutil



def real_load_mat(datatype,res,dir):
    """
    This function should load the real data and history data,
    the file format should be the same like:"omni_flowspeed_1min.mat"
    ACE data need to do a timeshift and calculate the

    @param datatype: The name of index :AE, SYMH, flowspeed, pressure, Bz_gsm
    @param res: resolution of the data,like 1 min
    @param dir: direction
    @return: data
    """
    dir_real = dir
    file_mat = dir_real + 'real_' + datatype + '_' + str(res) + 'm.mat'

    if (os.path.isfile(file_mat)):
        data = loadmat(file_mat)
        return data
    else:
        raise IOError('Real file does not exist:' + file_mat)

    pass


#
# def omni_load_mat(datatype, res, dir):
#     ''' Load OMNI data. '''
#     dir_omni = dir + '//omni//'
#     file_mat = dir_omni + 'omni_' + datatype + '_' + str(res) + 'min.mat'
#
#     if (os.path.isfile(file_mat)):
#         data = loadmat(file_mat)
#         return data
#     else:
#         raise IOError('Omni file does not exist:' + file_mat)

def fill_gap(list_input, max_gap):
    """
    interpolate the gap in the list
    :param list_input: The input list
    :param max_gap: the max gap allowed to be filled
    :return: new list
    """
    # Find where is the gap

    n = np.concatenate(([False], np.isnan(list_input), [False]))
    index = np.where(n[:-1] != n[1:])[0]

    # Find data turning point

    if len(index) > 1:
        for i in range(len(index) - 1):
            if np.isnan(list_input[index[i]]) and (index[i + 1] - index[i]) <= max_gap and (
                    index[i] > 0 and index[i + 1] < len(list_input)):  # fill the gap now

                difference = list_input[index[i + 1]] - list_input[index[i] - 1]
                gap_length = index[i + 1] - index[i]
                for gap_count in range(index[i + 1] - index[i]):
                    list_input[index[i] + gap_count] = (difference / (gap_length + 1)) * (gap_count + 1) + \
                                                       list_input[index[i] - 1]

    return list_input

def datenum_to_datetime(date_num):
    """
    Convert Matlab datenum into Python datetime.
    :param datenum: Date in datenum format
    :return:        Datetime object corresponding to datenum.
    """
    days = date_num % 1
    return datetime.fromordinal(int(date_num)) \
           + timedelta(days=days) \
           - timedelta(days=366)

def datetime_to_datenum(date_time):
    """
    Convert Python datetime to Matlab datenum
    :param date_time: Datetime object
    :return: date_num in float
    """
    # change date to timestamp
    mdn = date_time + timedelta(days=366)
    frac = (date_time - datetime(date_time.year, date_time.month, date_time.day, 0, 0, 0)).seconds / (
            24.0 * 60.0 * 60.0)
    return mdn.toordinal() + frac

def time_series_init(start_time, end_time, time_res):
    """
    Make a time series in (datetime, datenum)
    :param start_time: datetime of start time
    :param end_time: datetime of end time
    :param time_res: time resolution in min
    :return: time_date, time_datenum
    """
    time_resolution = timedelta(minutes=time_res)
    date_series_np = np.arange(start_time, end_time, time_resolution)
    date_series = date_series_np.astype(datetime)
    datenum_series = np.array([datetime_to_datenum(date_series[i]) for i in range(0, len(date_series))])
    return date_series, datenum_series

# def _extract_from_package(resource):
#     data = pkgutil.get_data('tempocnn', resource)
#     with tempfile.NamedTemporaryFile(prefix='model', suffix='.h5', delete=False) as f:
#         f.write(data)
#         name = f.name
#     return name

class FluxPredict(object):
    def __init__(self,
                 model_dir=CONFIG['model_dir'],
                 data_dir = CONFIG['real_data_dir'],
                 model_name = 'DL_15',
                 start_time=datetime(2017, 2, 1, 8, 0),
                 end_time=datetime(2017, 2, 11, 12, 0),
                 interp_time_res = 1,
                 channel=1,
                 data_type='rbsp',  # Another possible data_type is 'mageis'
                 ##############################
                 real_flag = True,

                 model_description='This is a 15 day history test model for radiation belt',
                 mindex=['ae_Boyel','dst','flowspeed','flow_pressure','bz_gsm'],
                 mres=np.array([15, 60, 1, 1, 1], dtype=np.int),
                 mwindow=np.array([180, 180, 180, 180, 180], dtype=np.int),
                 gap_max=np.array([100, 10, 144, 144, 144], dtype=np.int),
                 mdlag=np.array([180, 180, 180, 180, 180], dtype=np.double),
                 mlag0=np.array([0, 0, 0, 0, 0], dtype=np.double),
                 mlag1=np.array([21600, 21600, 21600, 21600, 21600], dtype=np.double),
                 L_range=[3, 6],
                 L_res=0.25,
                 L_num=12
                 ):
        self.data_dir = data_dir
        self.model_dir = model_dir
        self.start_time = start_time
        self.end_time = end_time
        self.channel = channel
        self.data_type = data_type
        self.model_name = 'DL_15'
        self.interp_time_res = interp_time_res
        self.X_input = None
        self.y_test = None
        self.L_range = L_range
        self.L_res = L_res
        self.L_num = L_num

        # Model information
        # Predict flag ,it true, means the real time data
        self.predict_flag = real_flag




        # model parameters
        self._model_name = model_name
        self._model_description = model_description
        self._mres = mres
        self._mindex = mindex
        self._mwindow = mwindow
        self._gap_max = gap_max
        self._mdlag = mdlag
        self._mlag0 = mlag0
        self._mlag1 = mlag1

        self._mwindow_index = (mwindow / mres).astype(int)
        self._fill_gap_flag = True

    def load_input(self):
        """
        This function is for loading the input from omni data
        @return: The input of model without time
        """
        #################################



        print('Loading input...')
        nindex = np.array([(self._mlag1[i] - self._mlag0[i]) / self._mdlag[i] + 1
                           for i in range(len(self._mdlag))]).astype(np.int)

        idx0 = np.append(0, np.cumsum(nindex)).astype(int)

        # Make a time series based on start_time and end_time
        self.total_tdata, self.total_tdata_datenum = time_series_init(self.start_time, self.end_time,
                                                            self.interp_time_res)


        ndata = len(self.total_tdata_datenum)
        ensem = np.empty([ndata, np.sum(nindex).astype(int)], dtype=np.double)



        for iidx, (index, res, window, gap_max) in enumerate(
                zip(self._mindex, self._mres, self._mwindow_index, self._gap_max)):
            ###########################################
            # if self.predict_flag:
            #     data = real_load_mat(index,res,self.data_dir)
            # else:
            #     data = omni_load_mat(index, res, self.data_dir)
            data = real_load_mat(index, res, self.data_dir)
            lags = np.linspace(self._mlag0[iidx], self._mlag1[iidx], num=nindex[iidx])
            # print(nindex,lags)
            mean_data_raw = data['data'].squeeze()
            time_raw = data['tdata'].squeeze()
            # change the omni time range for time history
            index_new = np.where(time_raw > (self.total_tdata_datenum[0] - 16))
            time_raw = time_raw[index_new]
            mean_data_raw = mean_data_raw[index_new]
            weights = np.ones(window) / window
            if self._fill_gap_flag:
                mean_data_temp = fill_gap(mean_data_raw, gap_max)
                mean_data_raw = mean_data_temp
                print('filling gap ...')

            mean_data = np.convolve(weights, mean_data_raw)[window - 1:-window + 1]

            mean_data_new = np.append(np.repeat(mean_data[0], window - 1), mean_data)
            fint = interp1d(time_raw, mean_data_new)

            for ilag, lag in enumerate(lags):

                ensem[:, ilag + idx0[iidx]] = fint(self.total_tdata_datenum - lag / 1440).flatten()

        self.input_total_data = ensem
        print('Load input success')



        # Now keep the data finite
        # remove the inf and NAN in the input value

        t_temp = np.expand_dims(self.total_tdata_datenum,axis=1)
        print('keeping finite...')
        TOTAL = np.concatenate((t_temp, self.input_total_data), axis=1)
        TOTAL_nan = TOTAL[~(np.isnan(TOTAL).any(axis=1))]
        TOTAL_new = TOTAL_nan[~(np.isinf(TOTAL_nan).any(axis=1))]
        print('keeping finite finished')
        self.total_tdata_datenum_finite = TOTAL_new[:, 0]
        self.X_input = TOTAL_new[:, 1:(self.input_total_data).shape[1] + 1]
        self.total_tdata_finite = np.array([datenum_to_datetime(self.total_tdata_datenum_finite[i])
                                            for i in range(0, len(self.total_tdata_datenum_finite))])

        return self.X_input

    def get_flux(self):
        """
        Use X_input to calculate the flux
        @return: y_output in 12 dimension
        """
        if self.X_input is None:
            raise ValueError('Please first run load_input')
        # Load the machine learning model
        model_name = self.model_dir + self.model_name + '_model.h5'

        orbis_model = tf.keras.models.load_model(model_name)
        print('calculating predict flux...')
        self.y_test = np.array(orbis_model(self.X_input))
        print('Success')
        return self.y_test

    def get_dataframe(self):
        """
        Combine the time and make an matrix
        @return: A panda dataframe
                Time    L_bin_0    L_bin_1 .... L_bin_11
        """
        if self.y_test is None:
            raise ValueError('Please first run load_input then run get_flux')

        # name_list = ['L_bin' + str(i) for i in range(self.L_num)]
        # orbis_dataFrame = pd.DataFrame(data=self.y_test, index=self.total_tdata_finite,columns = name_list)
        # print(orbis_dataFrame)
        print(self.y_test.shape)
        orbis_data = self.y_test.flatten()
        orbis_time = np.repeat(self.total_tdata_finite,self.L_num,axis = 0)
        orbis_L = np.tile(np.arange(self.L_range[0],self.L_range[1],self.L_res),len(self.y_test))
        orbis_dataFrame = pd.DataFrame({'time':orbis_time,'L':orbis_L,'eflux':orbis_data})
        return orbis_dataFrame

    def make_predict_plot(self,save = True):
        """
        Make the predict plot
        @param save: The flag of whether save the flag

        """
        if self.y_test is None:
            raise ValueError('Please first run load_input then run get_flux')
        eflux_data = np.transpose(self.y_test)
        fig,ax = plt.subplots(figsize = (12,8))
        norm = plt.Normalize(0.1, 6.5)
        L_data = np.arange(self.L_range[0],self.L_range[1]+self.L_res,self.L_res)
        time_mesh,L_mesh = np.meshgrid(self.total_tdata_finite, L_data)
        time_now = datetime.utcnow()
        predict_plot = ax.pcolormesh(time_mesh,L_mesh,eflux_data,cmap='jet',norm=norm)
        color_axis_ml = inset_axes(ax, width="1%",  # width = 5% of parent_bbox width
                                   height="100%",
                                   loc='lower left',
                                   bbox_to_anchor=(1.05, 0., 1, 1),
                                   bbox_transform=ax.transAxes,
                                   borderpad=0,
                                   )
        ax.set_ylabel('L_shell')
        Channel_name = 'Channel_' + str(self.channel)
        ax.set_title('ORBIS_Prediction  ' + Channel_name)
        fig.colorbar(predict_plot,cax = color_axis_ml,label = "log(Eflux)")
        # time_plot_range = timedelta(days=1)
        # ax.set_xlim(self.total_tdata_finite[0],time_now + time_plot_range)
        ax.vlines(x = time_now,ymin = 3,ymax = 6,label='utcnow')
        ax.legend()
        fig.autofmt_xdate()

        if save:
            fig_name = self.model_name +'_'+Channel_name + '.png'
            plt.savefig(fig_name)
        fig.show()

    def download_model(self):
        """
        Load the model from ???

        @return:
        """
        # if not os.path.exists(CONFIG['model_dir']):
        #     os.makedirs(CONFIG['model_dir'])
        #   Then download the model
        if os.path.exists(CONFIG['model_dir'] + self.model_name +'model.h5'):
            print(self.model_name +' model exist!')
        else:
            raise ValueError('Model not exist!')

def get_realtime_flux(utcnow = datetime.utcnow(),
                  time_range = 3,
                  model_name = 'DL_15',noupdate = False,dst_version = 'dst',makeplot = True):
    """

    @param utcnow: datetime of utcnow or other datetime specific
    @param time_range: date of the time history
    @param model_name: name of the model
    @param noupdate: if no update is true, will use current files to calculate the
                    flux value.
    @param dst_version: 'dst' or 'dst_kyoto'
    @return: a panda data frame
    """
    time_now = utcnow
    start_time = time_now - timedelta(days=time_range)
    #end_time = time_now
    #orbis = FluxPredict(start_time=start_time, end_time=end_time, model_name=model_name)
    orbis = FluxPredict(start_time=start_time, model_name=model_name)
    orbis.ae_frame = ae.load(time_now=time_now, no_update=noupdate)
    orbis.swepam_frame = ace.load(time_now=time_now, instrument='swepam', no_update=noupdate)
    orbis.mag_frame = ace.load(time_now=time_now, instrument='mag', no_update=noupdate)
    if dst_version is 'dst':
        orbis.dst_frame = dst.load(time_now=time_now, no_update=noupdate)
    else:
        orbis.dst_frame = dst_kyoto.load()

    # get the predict end time
    end_time  = (np.min([orbis.ae_frame.index.values[-1],
          orbis.swepam_frame.index.values[-1],
          orbis.mag_frame.index.values[-1],
          orbis.dst_frame.index.values[-1]]))

    end_time_date = pd.Timestamp(end_time).to_pydatetime()

    orbis = FluxPredict(start_time=start_time, end_time=end_time_date, model_name=model_name)
    orbis.load_input()
    orbis.get_flux()
    orbis.predict_frame = orbis.get_dataframe()
    if makeplot:

        orbis.make_predict_plot()
    return orbis.predict_frame


    #orbis_input_data = orbis.load_input()


    # orbis.ae_frame = ae_frame
    # orbis.dst_frame = dst_frame
    # orbis.swepam_frame = swepam_frame
    # orbis.mag_frame = mag_frame


    #flux = orbis.get_flux()






# @Author : Donglai
# @Time   : 7/12/2020 2:36 AM
# @Email  : dma96@atmos.ucla.edu donglaima96@gmail.com
import os

CONFIG ={'local_data_dir':'ace_data/',
         'real_data_dir':'real_data/',
         'remote_data_dir':'ftp://ftp.swpc.noaa.gov/pub/lists/ace/'}




# @Author : Donglai
# @Time   : 7/12/2020 2:47 AM
# @Email  : dma96@atmos.ucla.edu donglaima96@gmail.com
import os
from.config import CONFIG
import shutil
import urllib.request as request
from contextlib import closing
import pandas as pd
from datetime import datetime,timedelta
from ..predict.realtime_read import pdframe_to_mat


def load(time_now = None,time_history = 30,no_update  = False):
    """
    This funtion is load the ae real time data from 'http://mms.rice.edu/realtime/'
    @param time_history:
    @return: pdframe
    """
    if not os.path.exists(CONFIG['local_data_dir']):
        os.makedirs(CONFIG['local_data_dir'])

    file_name = 'archive_Rice_ae_1hr.txt'
    url = CONFIG['remote_data_dir'] + file_name
    file = CONFIG['local_data_dir'] + file_name

    with closing(request.urlopen(url)) as r:
        if os.path.exists(file) and no_update is True:
            print('File exist:',file)
        else:

            with open(file, 'wb') as f:
                shutil.copyfileobj(r, f)
                print('Downloading  %s' % file)

    AE_all = pd.read_csv(file,sep='\s+',parse_dates=[[5, 1, 2, 3]],header= None)
    AE_frame = AE_all.iloc[:,[0,3,4,5]]
    AE_frame.columns = ['date', 'ae_Boyel', 'ae_Ram', 'ae_Newell']

    # Add one hor prediction
    AE_predict_base = AE_frame[-1:]
    for i in range(4):
        predict = AE_frame[-1:].copy()
        predict['date'] += (i+1)*timedelta(minutes = 15)
        AE_predict_base = AE_predict_base.append(predict,ignore_index = True)




    AE_frame_new = AE_frame.append(AE_predict_base[1:],ignore_index = True)

    AE_frame_new.set_index('date',inplace = True, drop = True)

    if time_now is None:
        date_now = datetime.utcnow()
    else:
        date_now = time_now


    date_str = (date_now - timedelta(days = time_history)).strftime("%Y%m%d")
    AE_select = AE_frame_new[date_str:]
    pdframe_to_mat(AE_select,real_dir=CONFIG['real_data_dir'],res='15m')
    print('Finished')
    return AE_select




# @Author : Donglai
# @Time   : 7/20/2020 3:55 PM
# @Email  : dma96@atmos.ucla.edu donglaima96@gmail.com
import os


CONFIG ={'local_data_dir':'dst_data/',
         'real_data_dir':'real_data/',
         'remote_data_dir': 'https://unifieddatalibrary.com',
         'services': '/sfm/rest'}


# @Author : Donglai
# @Time   : 7/12/2020 2:02 AM
# @Email  : dma96@atmos.ucla.edu donglaima96@gmail.com

from datetime import datetime,timedelta
import numpy as np
import os
from scipy.io import savemat,loadmat

def datetime_to_datenum(date_time):
    """
    Convert Python datetime to Matlab datenum
    :param date_time: Datetime object
    :return: date_num in float
    """
    # change date to timestamp
    mdn = date_time + timedelta(days=366)
    frac = (date_time - datetime(date_time.year, date_time.month, date_time.day, 0, 0, 0)).seconds / (
            24.0 * 60.0 * 60.0)
    return mdn.toordinal() + frac


def pdframe_to_mat(pdframe, real_dir, res='1min'):
    """
    Change pdframe to a mat file
    the pdframe is like:
                         proton_density  flowspeed  flow_pressue
date
2020-05-29 00:00:00             3.8      291.1      0.644018
2020-05-29 00:01:00             3.6      292.7      0.616848
2020-05-29 00:02:00             3.8      293.0      0.652452
2020-05-29 00:03:00             4.0      292.6      0.684918
2020-05-29 00:04:00             4.1      293.2      0.704923

    # Change the datetime to datenum, I always to expand dims in matlab
        # In our dataset, all the datafile is composed of 'tdata' and 'data'
        # Each parameter has one name like'bz_gsm', 'flowspeed'...
        # The format is like source + name + resolution.mat
        # For OMNI it's like : 'omni_bz_gsm_1min.mat'
        # This time it's real so:'real_bz_gsm_1min.mat'
    @param pdframe:pdframe
    @param matname:the target mat file name
    @param dir:directory
    @return:1
    """
    tdata_datetime = pdframe.index.to_pydatetime()
    tdata = np.expand_dims(np.array([datetime_to_datenum(tdata_datetime[i])
                                     for i in range(0, len(tdata_datetime))]), axis=1)
    column_list = pdframe.columns.values

    if not os.path.exists(real_dir):
        os.makedirs(real_dir)
    for column_name in column_list:
        # The dir have a '/'
        file_name =real_dir + 'real_' + column_name + '_' + res + '.mat'
        data = np.expand_dims(pdframe[column_name].to_numpy(), axis=1)
        savemat(file_name, {'tdata': tdata, 'data': data})




# @Author : Donglai
# @Time   : 7/12/2020 2:47 AM
# @Email  : dma96@atmos.ucla.edu donglaima96@gmail.com
import os
import numpy as np
from.config import CONFIG
from datetime import datetime, timedelta
import shutil
import urllib.request as request
from contextlib import closing
import pandas as pd
from scipy.io import savemat
from ..predict.realtime_read import pdframe_to_mat



def load(time_now = None,
         instrument = 'mag',
         res = '1m',

         no_update = False,

         time_history = 30,
         shift_min = 40):
    """
    This function loads data from the ACE realtime website
    @param start_time: start time of the trange(np.datetime64)
    @param end_time: end time of the trange(np.datetime64)
    @param instrument: 'mag' or 'swepam'
    @param res:'1min' or '5min'
    @param downloadonly: bool; True is only download the data
    @param shift_min: shift minutes to bow shock nose point
    @return:
    """
    if not os.path.exists(CONFIG['local_data_dir']):
        os.makedirs(CONFIG['local_data_dir'])
    if time_now is None:
        date_now = datetime.utcnow()
    else:
        date_now = time_now

    date_str_list = [(date_now - timedelta(days=x)).strftime("%Y%m%d")
                         for x in range(time_history)]
    date_str_list.sort()


    ###########
    # real time is false:
    #########

    name_suffix = 'ace_' + instrument + '_' + res +'.txt'

    filenames = [CONFIG['local_data_dir'] + x + '_' + name_suffix for x in date_str_list]
    urls = [CONFIG['remote_data_dir'] + x + '_'+ name_suffix for x in date_str_list]
    filenames.append(CONFIG['local_data_dir'] + name_suffix )
    urls.append(CONFIG['remote_data_dir'] + name_suffix)

    # download the file
    if no_update is True:
        pass
    else:
        no_update_list = filenames[:-2]
        for url,file in zip(urls,filenames):
            if os.path.exists(file) and file in no_update_list:
                print('File exist:',file)
                continue
            with closing(request.urlopen(url)) as r:
                with open(file, 'wb') as f:
                    shutil.copyfileobj(r, f)
                    print('Downloaded  %s'%file)

    # READ the file
    if instrument is 'mag':
        mag_pdframe_list = []
        for f in filenames:
            daily = pd.read_csv(f, skiprows=20, header=None, sep='\s+', na_values='-999.9',
                                parse_dates=[[0, 1, 2, 3]])
            daily_bz = daily.iloc[:, [0, 6]]
            daily_bz.columns = ['date', 'bz_gsm']
            # Add 40 minutes to simulate the bow shock nose point
            daily_bz.loc[:,'date'] +=timedelta(minutes=shift_min)

            daily_bz.set_index('date', inplace=True, drop=True)
            mag_pdframe_list.append(daily_bz)

        mag_all_raw = pd.concat(mag_pdframe_list)
        # Remove the conflict values in real data and the value in the latest'YYYYHHMM' file
        mag_all = mag_all_raw.groupby(mag_all_raw.index).first()
        pdframe_to_mat(mag_all,real_dir=CONFIG['real_data_dir'],res=res)
        return mag_all
    elif instrument is 'swepam':
        swepam_pdframe_list = []
        for f in filenames:
            daily = pd.read_csv(f, skiprows=18, header=None, sep='\s+', na_values='-9999.9',
                                parse_dates=[[0, 1, 2, 3]])
            daily_SW = daily.iloc[:, [0, 4, 5]]
            daily_SW.columns = ['date', 'proton_density', 'flowspeed']
            # Add timeshift to simulate the bow shock nose point
            daily_SW.loc[:, 'date'] += timedelta(minutes=shift_min)
            daily_SW.set_index('date', inplace=True, drop=True)
            swepam_pdframe_list.append(daily_SW)

        swepam_all_raw = pd.concat(swepam_pdframe_list)
        swepam_all = swepam_all_raw.groupby(swepam_all_raw.index).first()
        swepam_all['flow_pressure'] = 2e-6 * swepam_all['proton_density'] * (swepam_all['flowspeed']) ** 2
        pdframe_to_mat(swepam_all,real_dir=CONFIG['real_data_dir'],res=res)
        return swepam_all












"""A setuptools based setup module."""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open

setup(
    name='orbis-new',
    version='0.0.2',
    description='Real time ORBIS data',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/donglai96/ORBIS_new',
    packages=find_packages(),
    author='Donglai Ma',
    author_email='dma96@atmos.ucla.edu',
    license='MIT',
    classifiers=['Development Status :: 4 - Beta',
                 'Intended Audience :: Science/Research',
                 'Topic :: Scientific/Engineering',
                 'License :: OSI Approved :: MIT License',
                 'Programming Language :: Python :: 3.7',
                 ],
    keywords='predict radiation-belt',

    #packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    #install_requires=['numpy','mechanize','tensorflow >=2.0','scipy','pandas'],
    python_requires='>=3.5',
    include_package_data=True,
)


# @Author : Donglai
# @Time   : 7/20/2020 3:55 PM
# @Email  : dma96@atmos.ucla.edu donglaima96@gmail.com

from tkinter import *
# from functools import partial
#
# import argparse
# import time
import glob
# import traceback
import requests
import json
from datetime import datetime,timedelta
from requests.auth import HTTPDigestAuth, HTTPBasicAuth
from.config import CONFIG
import os
import numpy as np
import pandas as pd
from ..predict.realtime_read import pdframe_to_mat

def download_file(url,username,password):
    # Splits the URL to Get Filename

    local_filename = CONFIG['local_data_dir']+url.split('/')[-1]
    print(local_filename)
    # Downloads the File

    r = requests.get(url, stream=True, auth=HTTPBasicAuth(username, password))

    with open(local_filename, 'wb') as f:

        for chunk in r.iter_content(chunk_size=1024):

            if chunk:
                f.write(chunk)

    return local_filename


def load(time_now = None,time_history = 30, no_update = False):
    if not os.path.exists(CONFIG['local_data_dir']):
        os.makedirs(CONFIG['local_data_dir'])

    # Get the destination sub_folder name, which is in sub_folder
    if time_now is None:
        date_now = datetime.utcnow()
    else:
        date_now = time_now

    current_time = date_now
    print(current_time)

    # Convert Time to Most Current JB2008 File Push
    if int(current_time.minute) < 58:
        current_time = current_time - timedelta(hours=1)
    current_time = str(current_time)
    yr = current_time[0:4]
    mon = current_time[5:7]
    day = current_time[8:10]

    # Build UDL Path to JB2008 Data
    sub_folder = '/SupportingData/SGI/JB2008/' + yr + '/' + mon + '/' + day + '/'
    #print(sub_folder + '\n')



    if no_update is False:
        # Generate a widget to get username and password
        print('Please input your username and password!')
        root = Tk()
        nameVar = StringVar()
        passVar = StringVar()

        def getName():
            root.destroy()

        root.geometry('300x150')
        root.title('Unified Data Library')
        userLabel = Label(root, text="Enter Username")
        username = Entry(root, bd=5, textvariable=nameVar)
        pwLabel = Label(root, text="Enter Password")
        password = Entry(root, bd=5, textvariable=passVar, show='*')
        submit = Button(root, text="Submit", command=getName)

        userLabel.pack()
        username.pack()
        pwLabel.pack()
        password.pack()
        submit.pack(side=BOTTOM)
        root.mainloop()

        username = nameVar.get()
        password = passVar.get()
        # print(username,password)

        # After get the username and password, download the data

        host = CONFIG['remote_data_dir']
        services = CONFIG['services']

        url = host + services + '/list' + sub_folder

        headers = {'Content-type': 'application/json'}

        # Gets File List

        r = requests.get(url, headers=headers, auth=HTTPBasicAuth(username, password))

        if (r.ok):

            d = json.loads(r.text)
            if len(d) == 0:
                return
            for index in [-4, -3, -2, -1]:
                if (d[index]['type'] == 'file'):
                    if 'DST' in str(d[index]['attributes']['name']):
                        dst_file_name = download_file(host + services + '/downloadFile' + d[index].get('id'),
                                      username,password)
                        print(type(dst_file_name))
        else:
            r.raise_for_status()
            dst_file_name = None
    else:
        dst_files = CONFIG['local_data_dir'] + '*DST*'
        if len(glob.glob(dst_files))>0:

            dst_file_name = glob.glob(dst_files)[-1]
            print(dst_file_name)
        else:
            dst_file_name = None


    if type(dst_file_name) is str:

        dst_f = open(dst_file_name, 'r')
    else:
        raise TypeError('dst file not exist!')

    data = dst_f.readlines()

    time_array = []

    dst_array = []

    for i, dst_line in enumerate(data):

        year = int(dst_line[3:5])

        if (year <= 99 and year >= 96):
            year = year + 1900

        if (year >= 00 and year <= 95):
            year = year + 2000

        month = int(dst_line[5:7])

        day = int(dst_line[8:10])

        for h in range(24):
            time_array.append(datetime(year, month, day, hour=h))

            dst_data = int(dst_line[h * 4 + 20:h * 4 + 24])
            dst_array.append(dst_data)

    date_all = np.array(time_array)
    dst_all = np.array(dst_array)
    start_time = datetime.utcnow() - timedelta(days=time_history)
    end_time = datetime.utcnow() + timedelta(hours=2)

    index = np.where((date_all > start_time) & (date_all < end_time))

    dst_frame = pd.DataFrame({'date': date_all[index], 'dst': dst_all[index]})
    dst_frame.set_index('date', inplace=True, drop=True)
    pdframe_to_mat(dst_frame, real_dir=CONFIG['real_data_dir'], res='60m')

    # dst_mean = round(dst_data[0:24].mean())

    # dst_matrix.append(dst_data)

    # =np.append(dst_matrix,dst_data,axis=0)

    # if( abs(dst_data[24] - dst_mean) > 1.0 and (dst_data[24] != 9999)):

    #    print(i,dst_mean,dst_data[24],dst_line)

    return dst_frame




# def load(dst_file,time_history = 30):
#
#
#     dst_f = open(dst_file, 'r')
#
#     data = dst_f.readlines()
#
#     time_array = []
#
#     dst_array = []
#
#     for i, dst_line in enumerate(data):
#
#         year = int(dst_line[3:5])
#
#         if (year <= 99 and year >= 96):
#             year = year + 1900
#
#         if (year >= 00 and year <= 95):
#             year = year + 2000
#
#         month = int(dst_line[5:7])
#
#         day = int(dst_line[8:10])
#
#         for h in range(24):
#
#             time_array.append(datetime(year, month, day,hour=h))
#
#             dst_data =int(dst_line[h * 4 + 20:h * 4 + 24])
#             dst_array.append(dst_data)
#
#
#     date_all = np.array(time_array)
#     dst_all = np.array(dst_array)
#     start_time = datetime.utcnow() - timedelta(days=time_history)
#     end_time = datetime.utcnow() + timedelta(hours=2)
#
#     index = np.where((date_all>start_time) & (date_all < end_time))
#
#
#     dst_frame= pd.DataFrame({'date':date_all[index],'dst':dst_all[index]})
#     dst_frame.set_index('date',inplace = True, drop = True)
#     pdframe_to_mat(dst_frame,real_dir=CONFIG['real_data_dir'],res='60m')
#
#         # dst_mean = round(dst_data[0:24].mean())
#
#         #dst_matrix.append(dst_data)
#
#         # =np.append(dst_matrix,dst_data,axis=0)
#
#         # if( abs(dst_data[24] - dst_mean) > 1.0 and (dst_data[24] != 9999)):
#
#         #    print(i,dst_mean,dst_data[24],dst_line)
#
#     return dst_frame